// Timings
export const SECOND = 1000;
export const MINUTE = 60 * SECOND;
export const HOUR = 60 * MINUTE;
export const DAY = 24 * HOUR;
export const WEEK = 7 * DAY;
export const MONTH = 30 * DAY;
export const YEAR = 365 * DAY;

/**
 * Maximum amount of time to back off requests for (in seconds)
 * @type {number}
 */
export const BACKOFF_CAP = 10;

/**
 * Amplification factor to increase deltas between backoff timers for requests
 * @type {number}
 */
export const BACKOFF_BASE = 1;

/**
 * Maximum number of requests to pack into a single multi request
 * @type {number}
 */
export const REQUESTS_PER_MULTI = 10;

/**
 * Standard amount of time to retry a failing request for
 * @type {number}
 */
export const STANDARD_RETRY = 30 * MINUTE;

/**
 * Standard amount of time to cache a request or object response
 * @type {number}
 */
export const STANDARD_TTL = DAY;

/**
 * Amount of time to wait between autoupdate check-ins
 * @type {number}
 */
export const SOFT_AUTOUPDATE_FREQ = DAY;

/**
 * Amount of time between individual page views before starting a new visit.
 * This is a Google Analytics standard amount of time
 * @type {number}
 */
export const ANALYTICS_MAX_VISIT_DURATION = 30 * MINUTE;

// Known endpoints
export const AUTH_ME_ENDPOINT = '/wp/v2/users/me';
export const AUTH_JWT_TOKEN_GET_ENDPOINT = '/storyshare/v1/auth/jwt/token';
export const AUTH_JWT_TOKEN_EXPIRE_ENDPOINT = '/storyshare/v1/auth/jwt/expire';

// Actions
export const AUTH_REGISTER = 'ss/ath/reg';
export const AUTH_LOGGED_IN = 'ss/ath/log/in';
export const AUTH_LOGIN_CREDENTIALS = 'ss/ath/log/in/cre';
export const AUTH_LOGIN_JWT = 'ss/ath/log/in/jwt';
export const AUTH_LOGIN_AUTOMATIC = 'ss/ath/log/in/aut';
export const AUTH_LOGGED_OUT = 'ss/ath/log/out';
export const AUTH_CHECK_LOGGED_IN = 'ss/ath/tli';
export const AUTH_SET = 'ss/ath/set';

export const OBJECT_ADD = 'ss/obj/add';
export const OBJECT_DROP = 'ss/obj/drp';

export const ASSET_ADD = 'ss/ast/add';
export const ASSET_DROP = 'ss/ast/drp';

export const CONNECTION_SET = 'ss/con/set';
export const CONNECTION_REFRESH = 'ss/con/ref';

export const FETCH_SET = 'ss/fet/set';
export const FETCH_SET_HEADER = 'ss/fet/set/hdr';

export const QUEUE_ADD = 'ss/qu/add';
export const QUEUE_DEQUEUE = 'ss/qu/proc/deq';
export const QUEUE_DROP = 'ss/qu/drp';
export const QUEUE_ENQUEUE = 'ss/qu/proc/enq';
export const QUEUE_CLEAR_BACKOFF = 'ss/qu/clr/bof';
export const QUEUE_PROCESS = 'ss/qu/proc/run';
export const QUEUE_UPDATE = 'ss/qu/upd';

export const META_SET = 'ss/me/set';
export const META_CLEAR = 'ss/me/clr';

export const SCHEMA_FETCH = 'ss/sch/fet';
export const SCHEMA_SET = 'ss/sch/set';

export const KV_SET = 'ss/kv/set';
export const KV_MERGE = 'ss/kv/mrg';
export const KV_CLEAR = 'ss/kv/clr';
export const KV_REMOVE = 'ss/kv/rem';

export const PROMISE_ADD = 'ss/pr/add';
export const PROMISE_RESOLVE = 'ss/pr/res';
export const PROMISE_REJECT = 'ss/pr/rej';
export const PROMISE_CANCEL = 'ss/pr/can';

export const STORE_DEHYDRATE = 'ss/sto/de';

export const REBOOT = 'ss/reboot';

export const ANALYTICS_INIT = 'ss/ana/ini';
export const ANALYTICS_PAGEVIEW = 'ss/ana/pvw';
export const ANALYTICS_EVENT = 'ss/ana/evn';
export const ANALYTICS_SET = 'ss/ana/set';
export const ANALYTICS_RANDOMISE_VISIT = 'ss/ana/vran';
export const ANALYTICS_RANDOMISE_PAGEVIEW = 'ss/ana/pran';
export const ANALYTICS_CLEAR = 'ss/ana/clr';

export const ANALYTICS_TYPE = {
  GENERIC: 1,
  NOTIFICATION: 2,
  APP: 3,
  AUTH: 4,
  SOCIAL: 5,
  MEDIA: 6,
  SEARCH: 7,
  NAVIGATION: 8,
};

export const ANALYTICS_GOOGLE_PLATFORM = {
  TAG_MANAGER: 1,
  CORDOVA: 2,
};

export const NOTIFICATION_SET = 'ss/not/set';
export const NOTIFICATION_REFRESH = 'ss/not/ref';
export const NOTIFICATION_REGISTER = 'ss/not/reg';
export const NOTIFICATION_UNREGISTER = 'ss/not/ure';

export const EVENT_UNHANDLED_REJECTION = 'ss/evt/uhr';
export const EVENT_AUTH_LOGGED_IN = 'ss/evt/ath/li';
export const EVENT_AUTH_LOGGED_OUT = 'ss/evt/aut/lo';
export const EVENT_AUTH_IDENTITY_CHANGED = 'ss/evt/auth/ic';
export const EVENT_NOTIFICATION_RECEIVED = 'ss/evt/not/rcv';

export const EVENT_REGISTER = 'ss/evt/reg';
export const EVENT_UNREGISTER = 'ss/evt/unr';
export const EVENT_DISPATCH = 'ss/evt/dsp';

export const SPLASHSCREEN_SET = 'ss/spl/set';
export const SPLASHSCREEN_SHOW = 'ss/spl/sho';
export const SPLASHSCREEN_HIDE = 'ss/spl/hid';

export const AUTOUPDATE_INIT = 'ss/aut/ini';
export const AUTOUPDATE_SET = 'ss/aut/set';
export const AUTOUPDATE_CHECK = 'ss/aut/chk';
export const AUTOUPDATE_REDIRECT = 'ss/aut/red';
export const AUTOUPDATE_LOAD_ASSETS = 'ss/aut/la';

export const AUTOUPDATE_STATUS_DISABLED = 'ss/aut/sta/dis';
export const AUTOUPDATE_STATUS_IDLE = 'ss/aut/sta/idl';
export const AUTOUPDATE_STATUS_UPDATING = 'ss/aut/sta/upd';
export const AUTOUPDATE_STATUS_PENDING_RESTART = 'ss/aut/sta/pnd';
export const AUTOUPDATE_STATUS_ERROR = 'ss/aut/sta/err';

export const VERSION_REFRESH = 'ss/ver/ref';
export const VERSION_SET = 'ss/ver/set';
export const VERSION_APPCONTAINER = 'container';
export const VERSION_PACKAGENAME = 'package';
export const VERSION_AUTOUPDATE = 'app';

// Events
export const RESUME = 'ss/evt/resume';
export const PAUSE = 'ss/evt/pause';
export const OFFLINE = 'ss/evt/offline';
export const ONLINE = 'ss/evt/online';
export const UNLOAD = 'ss/evt/unload';

export const INIT = 'ss/init';
export const NOOP = 'ss/noop';

// Modules
export const ANALYTICS = 'analytics';
export const ASSET = 'asset';
export const AUTH = 'auth';
export const AUTOUPDATE = 'autoupdate';
export const CORDOVA = 'cordova';
export const FETCH = 'fetch';
export const NOTIFICATION = 'notification';
export const REQUEST = 'request';
export const SCHEMA = 'schema';
export const SPLASHSCREEN = 'splashscreen';
export const STORE = 'store';

// Platforms
export const PLATFORM_NODEJS = 'nodejs';
export const PLATFORM_CORDOVA = 'cordova';
export const PLATFORM_BROWSER = 'browser';
export const PLATFORM_CORDOVA_IOS = 'ios';
export const PLATFORM_CORDOVA_ANDROID = 'android';
export const PLATFORM_CORDOVA_UNKNOWN = 'unknown';
export const PLATFORM_UNKNOWN = 'unknown';

// Network types
export const NETWORK_BLUETOOTH = 'bluetooth';
export const NETWORK_CELLULAR = 'cellular';
export const NETWORK_ETHERNET = 'ethernet';
export const NETWORK_NONE = 'none';
export const NETWORK_WIFI = 'wifi';
export const NETWORK_WIMAX = 'wimax';
export const NETWORK_OTHER = 'other';
export const NETWORK_UNKNOWN = 'unknown';
export const NETWORK_SLOW_2G = 'slow-2g';
export const NETWORK_2G = '2g';
export const NETWORK_3G = '3g';
export const NETWORK_4G = '4g';

// Methods
export const METHOD_CREATE = 'create';
export const METHOD_READ = 'read';
export const METHOD_UPDATE = 'update';
export const METHOD_UPLOAD = 'upload';
export const METHOD_DELETE = 'delete';
export const METHOD_DOWNLOAD = 'download';
export const METHOD_MULTI = 'multi';

// Errors
export const ERROR_INTERNAL = 'internal';
export const ERROR_USAGE = 'usage';

// HTTP status codes
export const HTTP_OK = 200;
export const HTTP_CREATED = 201;
export const HTTP_BAD_REQUEST = 400;
export const HTTP_UNAUTHORIZED = 401;
export const HTTP_FORBIDDEN = 403;
export const HTTP_NOT_FOUND = 404;
export const HTTP_METHOD_NOT_ALLOWED = 405;
export const HTTP_CONFLICT = 409;
export const HTTP_GONE = 410;
export const HTTP_PRECONDITION_FAILED = 412;
export const HTTP_INTERNAL_SERVER_ERROR = 500;
export const HTTP_NOT_IMPLEMENTED = 501;

// Authentication

export const AUTH_STATUS_UNKNOWN = 'unknown';
export const AUTH_STATUS_PENDING = 'pending';
export const AUTH_STATUS_LOGGED_IN = 'loggedin';
export const AUTH_STATUS_LOGGED_OUT = 'loggedout';
export const AUTH_STATUS_RETRY = 'retry';
export const AUTH_STATUS_FAILURE = 'failure';

/**
 * The state of this request is not yet known or has been expired
 *
 * @type {string}
 */
export const STATUS_UNKNOWN = 'unknown';

/**
 * This request is ready to be inserted into the dispatch pipeline
 *
 * @type {string}
 */
export const STATUS_READY = 'ready';

/**
 * This request has succeeded
 *
 * @type {string}
 */
export const STATUS_SUCCESS = 'success';

/**
 * This request has failed
 *
 * @type {string}
 */
export const STATUS_FAILED = 'failed';

/**
 * This request is currently backing off due to an error
 *
 * @type {string}
 */
export const STATUS_BACKOFF = 'backoff';

/**
 * This request could not be completed, and is pending being checked for retry
 *
 * @type {string}
 */
export const STATUS_RETRY = 'retry';

/**
 * This request is in flight and waiting for server response
 *
 * @type {string}
 */
export const STATUS_INFLIGHT = 'inflight';
