import { append, defaultTo, filter, has, map, path } from 'ramda';

import { EVENT_DISPATCH, EVENT_REGISTER, EVENT_UNREGISTER } from './const';

export const register = (event, callback, once = false) => ({
  type: EVENT_REGISTER,
  event,
  callback,
  once,
});

export const unregister = (event, callback) => ({
  type: EVENT_UNREGISTER,
  event,
  callback,
});

export const generateEvent = (event, data) => ({
  type: EVENT_DISPATCH,
  event,
  data,
});

export const middleware = ({ getState, dispatch }) => (next) => (action) => {
  switch (action.type) {
    case EVENT_DISPATCH:
      const { event } = action;

      const handlers = defaultTo([], path(['event', event], getState()));

      map((handler) => {
        if (typeof handler.callback !== 'function') {
          return;
        }

        handler.callback(action.data);

        if (handler.once) {
          dispatch(unregister(event, handler.callback));
          delete handler.callback;
        }
      }, handlers);
      break;
  }

  return next(action);
};

export const reducer = (state = {}, { type, event, callback, once }) => {
  switch (type) {
    case EVENT_REGISTER:
      if (!has(event, state)) {
        state[ event ] = [];
      }

      state[ event ] = append({
        callback,
        once,
      }, state[ event ]);

      return state;

    case EVENT_UNREGISTER:
      if (!has(event, state)) {
        return state;
      }

      state[ event ] = filter((e) => e.callback !== callback, state[ event ]);

      return state;
  }

  return state;
};
