import { defaultTo, merge, omit, prop } from 'ramda';

import { KV_CLEAR, KV_REMOVE, KV_SET } from './const';
import { runPipeline } from './queue';

export const kvSet = (object) => ({
  type: KV_SET,
  object,
});

export const kvClear = () => ({
  type: KV_CLEAR,
});

export const kvRemove = (key) => ({
  type: KV_REMOVE,
  key,
});

export const middleware = ({ dispatch }) => (next) => (action) => {
  switch (action.type) {
    case KV_SET:
    case KV_CLEAR:
    case KV_REMOVE:
      runPipeline(dispatch);
  }

  return next(action);
};

export const reducer = (state = {}, action) => {
  switch (action.type) {
    case KV_SET:
      return merge(state, defaultTo({}, prop('object', action)));

    case KV_REMOVE:
      return omit([action.key], state);

    case KV_CLEAR:
      return {};
  }

  return state;
};
