import { merge, path } from 'ramda';

import { META_CLEAR, META_SET } from './const';

/**
 * Action creator to set a meta section of the store
 * @param {object} meta
 * @returns {{type: string, meta: *}}
 * @private
 */
export const set = (meta) => ({
  type: META_SET,
  meta,
});

/**
 * Selector to get a meta key from the state
 * @param {object} state
 * @param {string} key
 * @returns {*}
 * @private
 */
export const get = (state, key) => path(['meta', key], state);

export const reducer = (state = {}, action) => {
  switch (action.type) {
    case META_SET:
      return merge(state, action.meta);

    case META_CLEAR:
      return {};
  }

  return state;
};
