import { merge } from 'ramda';

import {
  AUTH_LOGGED_IN,
  NOTIFICATION_REFRESH,
  NOTIFICATION_REGISTER,
  NOTIFICATION_SET,
  NOTIFICATION_UNREGISTER,
} from './const';
import * as platform from './platforms/notification';

export const register = () => ({
  type: NOTIFICATION_REGISTER,
});

export const refresh = () => ({
  type: NOTIFICATION_REFRESH,
});

export const set = (meta) => ({
  type: NOTIFICATION_SET,
  meta,
});

/**
 * Action creator to set the notification status
 * @param {boolean} enabled - whether notifications are enabled
 * @returns {AnyAction}
 */
export const setEnabled = (enabled) => set({ enabled });

/**
 * Selector to read the notification status
 * @param {Store} state - store state
 * @returns {boolean} whether notifications are enabled
 */
export const getEnabled = (state) => !!state.notification.enabled;

export const middleware = (api) => (next) => (action) => {
  switch (action.type) {
    case AUTH_LOGGED_IN:
    case NOTIFICATION_REFRESH:
      platform.refresh(api);
      break;

    case NOTIFICATION_UNREGISTER:
      platform.unregister(api);
      break;

    case NOTIFICATION_REGISTER:
      platform.register(api);
      break;
  }

  return next(action);
};

export const reducer = (state = {}, action) => {
  switch (action.type) {
    case NOTIFICATION_SET:
      return merge(state, action.meta);
  }

  return state;
};
