import 'cross-fetch/polyfill';
import hash from 'object-hash';
import URI from 'urijs';

import { HTTP_OK, PLATFORM_BROWSER, PLATFORM_CORDOVA, PLATFORM_NODEJS } from '../const';
import { Promise } from '../promise';
import { writeFile } from '../storage';
import { downloadFile } from './storage/cordova';
import { platform, stripQuery } from '../utils';

/**
 * Download a resource
 * @param {string} url - URL to retrieve
 * @returns {Promise<{ result: { filename?: string }, response: Response }>}
 */
export const download = (url) => {
  switch (platform()) {
    case PLATFORM_BROWSER:
      return Promise.resolve({
        result: {},
        response: { status: HTTP_OK },
      });

    case PLATFORM_CORDOVA:
      return (async () => {
        const filename = await downloadFile(url);

        return {
          result: { filename },
          response: { status: HTTP_OK },
        };
      })();
    case PLATFORM_NODEJS:
      return (async () => {
        const response = await fetch(url);

        if (response.status !== HTTP_OK) {
          return {
            result: {},
            response,
          };
        }

        const filename = `${hash(stripQuery(url))}.${URI(url).suffix()}`;

        await writeFile(`assets/${filename}`, await response.arrayBuffer());

        return {
          result: { filename },
          response,
        };
      })();
  }
};
