import URI from 'urijs';

import { PLATFORM_BROWSER, PLATFORM_CORDOVA } from '../const';
import { show as showSplashScreen } from '../splashscreen';
import { platform } from '../utils';

/**
 * Reboots the application
 * @param {Store} store - Redux store
 * @param {string} target - location for navigation (defaults to `/`)
 * @returns {Promise} - Promise that will be resolved if the app can continue
 */
export const reboot = ({ dispatch }, target) => {
  switch (platform()) {
    case PLATFORM_BROWSER:
      target = target || '/';

      if (window.location.href === target) {
        window.location.reload();
      } else {
        window.location.href = target;
      }
      break;

    case PLATFORM_CORDOVA:
      dispatch(showSplashScreen());

      if (target) {
        if (window.location.href === target) {
          window.location.reload();
        } else {
          window.location.href = target;
        }
        return;
      }

      window.location.href = URI().fragment('').toString();
      break;
  }
};
