import { CORDOVA, PLATFORM_CORDOVA, SPLASHSCREEN } from '../const';
import { debug, warn } from '../debug';
import { setShowing } from '../splashscreen';
import { platform } from '../utils';

/**
 * Shows the splashscreen
 * @param {Store} store - Redux store
 */
export const show = ({ dispatch }) => {
  switch (platform()) {
    case PLATFORM_CORDOVA:
      if (!navigator.splashscreen) {
        warn(CORDOVA, 'Splash screen plugin is not installed');
        break;
      }

      debug(SPLASHSCREEN, 'Showing splash screen');
      navigator.splashscreen.show();
      dispatch(setShowing(true));
      break;

    default:
      dispatch(setShowing(true));
      break;
  }
};

/**
 * Hides the splashscreen
 * @param {Store} store - Redux store
 */
export const hide = ({ dispatch }) => {
  switch (platform()) {
    case PLATFORM_CORDOVA:
      if (!navigator.splashscreen) {
        warn(CORDOVA, 'Splash screen plugin is not installed');
        break;
      }

      debug(SPLASHSCREEN, 'Hiding splash screen');
      navigator.splashscreen.hide();
      dispatch(setShowing(false));
      break;

    default:
      dispatch(setShowing(false));
      break;
  }
};
