import { CORDOVA, PLATFORM_BROWSER, PLATFORM_CORDOVA, PLATFORM_NODEJS } from '../const';
import { warn } from '../debug';
import { platform } from '../utils';

/**
 * Encodes a string using the Base64 encoding scheme
 * @param {string} string - string to encode
 * @returns {string} Base64-encoded version of input string
 */
export const base64encode = (string) => {
  switch (platform()) {
    case PLATFORM_BROWSER:
    case PLATFORM_CORDOVA:
      return btoa(string);

    case PLATFORM_NODEJS:
      return Buffer.from(string).toString('base64');
  }
};

/**
 * Reads flags from an appropriate location
 * @returns {string|undefined}
 */
export const readFlags = () => {
  switch (platform()) {
    case PLATFORM_BROWSER:
    case PLATFORM_CORDOVA:
      return localStorage.storyshareConfig;

    case PLATFORM_NODEJS:
      return process.env.STORYSHARE_CONFIG;
  }
};

/**
 * Reads the connection type from navigator api
 * @returns {string} the connection type
 */
export const getConnectionType = () => {
  switch (platform()) {
    case PLATFORM_CORDOVA:

      if (!navigator.connection) {
        warn(CORDOVA, 'Connection plugin is not installed');
        break;
      }

      return navigator.connection.type;

    case PLATFORM_BROWSER:
      if (!navigator.connection) {
        break;
      }

      return navigator.connection.effectiveType || navigator.connection.type;
  }
};
