import Bluebird from 'bluebird';
import { append, defaultTo, map, merge, omit, path } from 'ramda';

import { PROMISE_ADD, PROMISE_CANCEL, PROMISE_REJECT, PROMISE_RESOLVE } from './const';

export const Promise = Bluebird.getNewLibraryCopy();

export const add = (hash, res, rej) => ({
  type: PROMISE_ADD,
  hash,
  res,
  rej,
});

export const resolve = (hash, result) => ({
  type: PROMISE_RESOLVE,
  hash,
  result,
});

export const reject = (hash, error) => ({
  type: PROMISE_REJECT,
  hash,
  error,
});

export const cancel = (hash) => ({
  type: PROMISE_CANCEL,
  hash,
});

export const middleware = ({ getState }) => (next) => (action) => {
  switch (action.type) {
    case PROMISE_RESOLVE:
      map(
        (f) => f(action.result),
        defaultTo([], path(['promises', action.hash, 'res'], getState())),
      );
      break;

    case PROMISE_REJECT:
      map(
        (f) => f(action.error),
        defaultTo([], path(['promises', action.hash, 'rej'], getState())),
      );
      break;
  }

  return next(action);
};

export const reducer = (state = {}, action) => {
  switch (action.type) {
    case PROMISE_ADD:
      return merge(state, {
        [ action.hash ]: {
          res: append(action.res, defaultTo([], path([action.hash, 'res'], state))),
          rej: append(action.rej, defaultTo([], path([action.hash, 'rej'], state))),
        },
      });

    case PROMISE_RESOLVE:
    case PROMISE_REJECT:
    case PROMISE_CANCEL:
      return omit([action.hash], state);
  }

  return state;
};
