import { authenticate, loggedOut } from './auth';
import { HTTP_UNAUTHORIZED, MINUTE, RESUME, SCHEMA, SCHEMA_FETCH, SCHEMA_SET, SECOND } from './const';
import { debug, error } from './debug';
import { externalFlag } from './utils';

export const fetch = () => ({
  type: SCHEMA_FETCH,
});

export const set = (schema) => ({
  type: SCHEMA_SET,
  schema,
});

export const reducer = (state = {}, action) => {
  switch (action.type) {
    case SCHEMA_SET:
      return action.schema;
  }

  return state;
};

export const middleware = ({ getState, dispatch }) => (next) => (action) => {
  switch (action.type) {
    case RESUME:
    case SCHEMA_FETCH:
      const store = getState();
      const { site } = store.meta;

      return site.req(site.self)
        .promise()
        .ttl(false)
        .retry(externalFlag('simulateOffline') ? SECOND : MINUTE)
        .read()
        .then((response) => {
          debug(SCHEMA, 'Schema updated');
          dispatch(set(response.body));

          return null;
        })
        .catch((err) => {
          switch (err.status) {
            case HTTP_UNAUTHORIZED:
              dispatch(loggedOut());

              return dispatch(authenticate())
                .then(() => dispatch(fetch()));
          }

          error(SCHEMA, 'Failed to fetch schema', err);
          throw err;
        });
  }

  return next(action);
};
