import { merge } from 'ramda';

import { SPLASHSCREEN_HIDE, SPLASHSCREEN_SET, SPLASHSCREEN_SHOW } from './const';
import * as platform from './platforms/splashscreen';

export const show = () => ({
  type: SPLASHSCREEN_SHOW,
});

export const hide = () => ({
  type: SPLASHSCREEN_HIDE,
});

export const set = (meta) => ({
  type: SPLASHSCREEN_SET,
  meta,
});

/**
 * Selector to read the splashscreen status
 * @param {Store} state - store state
 * @returns {boolean} whether the splashscreen is showing
 */
export const isShowing = (state) => !!state.splashscreen.showing;

/**
 * Action creator to set the splashscreen status
 * @param {boolean} showing - whether the splashscreen is showing
 * @returns {AnyAction}
 */
export const setShowing = (showing) => set({ showing });

export const middleware = (api) => (next) => (action) => {
  switch (action.type) {
    case SPLASHSCREEN_SHOW:
      platform.show(api);
      break;

    case SPLASHSCREEN_HIDE:
      platform.hide(api);
      break;
  }

  return next(action);
};

export const reducer = (state = { showing: false }, action) => {
  switch (action.type) {
    case SPLASHSCREEN_SET:
      return merge(state, action.meta);
  }

  return state;
};
