import { merge } from 'ramda';

import { VERSION_REFRESH, VERSION_SET } from './const';
import * as platform from './platforms/version';

/**
 * Action creator to store a component version
 * @param {string} component - component being analysed
 * @param {string} version - version to store
 * @returns {AnyAction}
 */
export const set = (component, version) => ({
  type: VERSION_SET,
  component,
  version,
});

export const refresh = () => ({
  type: VERSION_REFRESH,
});

/**
 * Selector to format version information
 * If `component` is omitted then an app version will be assembled
 * @param {string?} component - component to check
 * @param {Store} state - store state
 * @returns {string|undefined}
 */
export const getVersion = (component, state) => {
  if (component) {
    return state.version[ component ];
  }

  return platform.formatVersion(state);
};

export const middleware = (api) => (next) => (action) => {
  switch (action.type) {
    case VERSION_REFRESH:
      return platform.refresh(api);
  }

  return next(action);
};

export const reducer = (state = {}, action) => {
  switch (action.type) {
    case VERSION_SET:
      return merge(state, {
        [ action.component ]: action.version,
      });
  }

  return state;
};
